@@ -24,11 +24,11 @@ main (void)
       return 1;
     }
 
-  in_args[0].v_pointer = "domain";
-  in_args[1].v_pointer = "glib-print.c";
+  in_args[0].v_pointer = (gpointer)"domain";
+  in_args[1].v_pointer = (gpointer)"glib-print.c";
   in_args[2].v_int = 42;
-  in_args[3].v_pointer = "main";
-  in_args[4].v_pointer = "hello world";
+  in_args[3].v_pointer = (gpointer)"main";
+  in_args[4].v_pointer = (gpointer)"hello world";
 
   if (!g_function_info_invoke ((GIFunctionInfo *) base_info,
                                (const GIArgument *) &in_args,
