@@ -16,6 +16,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 #
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
 from . import ast
 from . import message
@@ -80,7 +84,7 @@ class IntrospectablePass(object):
 
         if not node.type.resolved:
             self._parameter_warning(parent, node,
-                                    "Unresolved type: %r" % (node.type.unresolved_string, ))
+                                    "Unresolved type: '%s'" % (node.type.unresolved_string, ))
             parent.introspectable = False
             return
 
