@@ -3,6 +3,11 @@
 # * Create a git tag
 # * Bump version in configure.ac and commit it
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import re
 import os
 import sys
@@ -35,13 +40,13 @@ for line in f:
     continue
   v = int(m.group(1))
   newv = v+1
-  print "Will update micro version from %s to %s" % (v, newv)
+  print("Will update micro version from %s to %s" % (v, newv))
   newf.write(micro_version_replace % (newv, ))
 newf.close()
 
 os.rename(configure_path + '.tmp', configure_path)
-print "Successfully wrote new 'configure.ac' with post-release version bump"
+print("Successfully wrote new 'configure.ac' with post-release version bump")
 
 args=['git', 'commit', '-m', "configure: Post-release version bump", configure_path]
-print "Running: %r" % (args, )
+print("Running: %r" % (args, ))
 subprocess.check_call(args)
