@@ -60,7 +60,12 @@ main (int argc, char *argv[])
 
   context = g_option_context_new ("");
   g_option_context_add_main_entries (context, options, NULL);
-  g_option_context_parse (context, &argc, &argv, &error);
+  if (!g_option_context_parse (context, &argc, &argv, &error))
+    {
+      g_fprintf (stderr, "failed to parse: %s\n", error->message);
+      g_error_free (error);
+      return 1;
+    }
 
   if (!input)
     {
@@ -75,11 +80,11 @@ main (int argc, char *argv[])
 
   for (i = 0; input[i]; i++)
     {
-      GError *error = NULL;
       const char *namespace;
       GMappedFile *mfile;
       GITypelib *typelib;
 
+      error = NULL;
       mfile = g_mapped_file_new (input[i], FALSE, &error);
       if (!mfile)
 	g_error ("failed to read '%s': %s", input[i], error->message);
